A number of specialized versions of the [TrainMatcher](trainmatcher.md) are used in `calng` pipelines.
The extensions provided in `ShmemTrainMatcher` (the device class for [group matchers](#group-matcher) and [full matchers](#full-matcher)) used within `calng` are:

1. They handle [shared memory handles](https://git.xfel.eu/karaboDevices/calng/-/wikis/ShmemProtocol)
    - This is necessary for group matchers
2. They allow stacking arrays from multiple sources
    - This is useful for connecting tools like OnDA to a full matcher
3. They support frame selection for online data reduction
    - Documentation for this feature is lacking


### Group matcher

For multi-module detectors, each detector module produces an independent data stream - to the control system, each module is essentially its own detector.
The data from one module is typically passed through one DAQ device from which it is then sent to a `calng` correction device.
As multiple data streams can be processed simultaneously, a **group** in `calng` denotes a set of modules which are *corrected* on the same compute node in the online cluster.
Fast detectors like AGIPD, DSSC, or LPD are typically split into four groups of four modules.
For slower detectors like JUNGFRAU, it may be feasible to process the entire detector wihin a sigle group.

Each group can optionally run a group matcher, which provides the fastest possible access to the output stream from the group.
If your online analysis relies only upon a subset of modules, configuring how the groups are divided (done via the manager) is the first step to set up an efficient output over a Karabo bridge.
See [manager notes](middlelayer.md#calibration-manager) for details on group matcher configuration.

Here is a screenshot of the overview scene from a group matcher in the wild:

![](../static/screenshot-group_matcher-overview.png "Screenshot: TrainMatcher - configured as group matcher - default scene")

### Full matcher

Important to note:

- Full matcher should connect to outputs of group matchers, not directly to correction devices
    - The group matchers will take care to send full data out (dereferencing shared memory handles from correction devices)
    - To configure sources, keep in mind that correction devices forward the source name they get from DAQs and that TrainMatcher needs to know source name and actual channel name in the format `[source]@[channel]`
    - Example source in `SPB_DET_AGIPD1M-1/CALNG/FULL_MATCHER`: `SPB_DET_AGIPD1M-1/DET/0CH0:xtdf@SPB_DET_AGIPD1M-1/CALNG/MATCH_G1:output`
- Full matcher is likely to hit [network limits](../data-rates.md)
	- Be aware of DAQ train stride and number of frames (or memory cells) coming from the detector
    - If possible, consider running the full matcher on the node where analysis is done
    - Configuring and testing full matcher with online analysis software should be done during commissioning (please contact calibration team ahead of time)

## Preview assemblers
As the name implies, a preview assembler (class name: [`DetectorAssembler`](../schemas/DetectorAssembler.md)) assembles preview images for full detectors by matching the data from the individual modules and applying a detector geometry to the data.
A preview assembler needs to contact a geometry device to get current detector geometry to use for assembly.
The [geometry classes](middlelayer.md#geometry-devices) and assembly methods directly use [EXtra-geom](https://extra-geom.readthedocs.io/en/latest/).


Preview assemblers provide multiple outputs:

- As a [TrainMatcher](trainmatcher.md), it has the typical "Pipeline Output" (`output`).
  This is unassembled, at full rate, and per source.
  Keep in mind that preview assemblers typically receive single-frame input - this is different to a [full matcher](#full-matcher).
- `assembledOutput` provides the assembled images at full speed.
  The output only contains `image.data` (single frame, assembled) and `trainId`.
  As the preview settings have not been applied to this data, this output may be useful for inspecting with small analysis devices.
- `preview.assembled.output` is the preview output, providing the image displayed on the default scene.
  See [preview](../concepts.md#preview) for details on how to configure the preview output.
  Keep in mind that frame reduction should be set [via the manager](middlelayer.md#assembled-preview-configuration) as it happens on the correction devices.

As a TrainMatcher, a preview assembler can also provide [Karabo bridge output](trainmatcher.md#karabo-bridge-output).
The `outputForBridgeOutput` parameter allows you to choose which of the three output channels the Karabo bridge output should mirror.

![](../static/screenshot-preview_assembler-preview.png "Screenshot: Preview assembler overview / preview scene")

Common issues:

- Misconfigured [`maxIdle`](../schemas/TrainMatcher.md#maxIdle); if too low (relative to DAQ train stride), it can cause [flickering](../troubleshooting.md#preview-is-flickering)
- Missing geometry can cause [preview to not update](../troubleshooting.md#preview-does-not-update)
