import enum


class KernelWarning(enum.Enum):
    INIT = "init"
    PROCESSING = "processing"
    PRESELECTION = "preselection"


class BaseArbiterKernel:
    def __init__(self, device, name, config):
        self._device = device
        self._name = name
        self.reconfigure(config)

    @staticmethod
    def extend_device_schema(schema, prefix):
        """Should add configurability to the arbiter (matcher) the kernel will be
        running in.  Prefix will include node name, so will typically be
        "frameSelection.kernels.{class.__name__}". The parent node will have been
        created already (unlike for correction addons), feel free to overwrite the
        description."""
        pass

    @staticmethod
    def extend_output_schema(schema, name):
        pass

    def consider(self, train_id, sources, num_frames, mask, out_hash):
        """Must return tuple of uint8 mask array of size equal to num_frames
        and base output hash or None."""
        raise NotImplementedError()

    def reconfigure(self, config):
        pass

    @property
    def geometry(self):
        """Same as BaseCorrectionAddon.geometry"""
        return self._device._geometry

    def on_new_geometry(self, geometry):
        """Same as BaseCorrectionAddon.on_new_geometry"""
        pass


class Assign(BaseArbiterKernel):
    def consider(self, tid, src, nframes, mask, out):
        return mask
