#include <cuda_fp16.h>

{{corr_enum}}

extern "C" {
	/*
	  Perform corrections: NONE or OFFSET
	  Take cell_table into account when getting correction values
	  Converting to float while correcting
	  Converting to output dtype at the end
	  Shape of input data: memory cell, 1, ss, fs
	  Shape of offset constant: fs, ss, memory cell
	*/
	__global__ void correct(const unsigned short* data, // shape: memory cell, 1, ss, fs
	                        const unsigned short* cell_table,
	                        const unsigned short input_frames,
	                        const unsigned short map_memory_cells,
	                        const unsigned char corr_flags,
	                        const float* offset_map,
	                        {{output_data_dtype}}* output) {
		const size_t ss_dim = 128;
		const size_t fs_dim = 512;

		const size_t memory_cell = blockIdx.x * blockDim.x + threadIdx.x;
		const size_t ss = blockIdx.y * blockDim.y + threadIdx.y;
		const size_t fs = blockIdx.z * blockDim.z + threadIdx.z;

		if (memory_cell >= input_frames || ss >= ss_dim || fs >= fs_dim) {
			return;
		}

		// note: strides differ from numpy strides because unit here is sizeof(...), not byte
		const size_t data_stride_fs = 1;
		const size_t data_stride_ss = fs_dim * data_stride_fs;
		const size_t data_stride_cell = ss_dim * data_stride_ss;
		const size_t data_index = memory_cell * data_stride_cell + ss * data_stride_ss + fs * data_stride_fs;
		float res = (float)data[data_index];

		const size_t map_stride_fs = 1;
		const size_t map_stride_ss = fs_dim * map_stride_fs;
		const size_t map_stride_cell = ss_dim * map_stride_ss;
		const size_t map_cell = cell_table[memory_cell];

		if (map_cell < map_memory_cells) {
			const size_t map_index = map_cell * map_stride_cell + ss * map_stride_ss + fs * map_stride_fs;
			if (corr_flags & OFFSET) {
				res -= offset_map[map_index];
			}
		}
		{% if output_data_dtype == "half" %}
		output[data_index] = __float2half(res);
		{% else %}
		output[data_index] = ({{output_data_dtype}})res;
		{% endif %}
	}
}
