import numpy as np


class ReductionOperation:
    def __init__(self, data, arg, mask, result_key):
        self.data = data
        self.arg = arg
        self.mask = mask
        self.result_key = result_key

    def __call__(self):
        result_arr = self.reduce()
        if result_arr is not None:
            self.data[f"reduce.{self.result_key}"] = result_arr

    def reduce(self):
        return None


class count(ReductionOperation):
    def reduce(self):
        arr = self.data[self.arg][self.mask]
        return np.sum(np.isfinite(arr), axis=0, initial=0)


class sum(ReductionOperation):
    def reduce(self):
        arr = self.data[self.arg][self.mask]
        return np.nansum(arr, axis=0, initial=0)


class sumsq(ReductionOperation):
    def reduce(self):
        arr = self.data[self.arg][self.mask]
        return np.nansum(arr * arr, axis=0, initial=0)


class max(ReductionOperation):
    def reduce(self):
        arr = self.data[self.arg][self.mask]
        return np.max(arr, axis=0, initial=-np.inf)


class min(ReductionOperation):
    def reduce(self):
        arr = self.data[self.arg][self.mask]
        return np.min(arr, axis=0, initial=np.inf)
