import numpy as np
import pytest

from calng import utils


def test_get_c_type():
    assert utils.np_dtype_to_c_type(np.float16) == "half"
    assert utils.np_dtype_to_c_type(np.float32) == "float"
    assert utils.np_dtype_to_c_type(np.float64) == "double"

    assert utils.np_dtype_to_c_type(np.uint8) == "unsigned char"
    assert utils.np_dtype_to_c_type(np.uint16) == "unsigned short"
    assert utils.np_dtype_to_c_type(np.uint32) in ("unsigned", "unsigned int")
    assert utils.np_dtype_to_c_type(np.uint64) == "unsigned long"

    assert utils.np_dtype_to_c_type(np.int8) == "char"
    assert utils.np_dtype_to_c_type(np.int16) == "short"
    assert utils.np_dtype_to_c_type(np.int32) == "int"
    assert utils.np_dtype_to_c_type(np.int64) == "long"


def test_options_enum():
    class SomeFunnyOptions(utils.OptionsEnum):
        A = 0
        B = 10
        any_identifier_should_work = -1

    options = SomeFunnyOptions.list_options()
    assert len(options) == 3
    parsed = [SomeFunnyOptions.parse_helpful_format(opt) for opt in options]
    assert list(SomeFunnyOptions) == parsed

    with pytest.raises(ValueError):
        SomeFunnyOptions.parse_helpful_format("I don't belong")

    # I don't have strong feelings about KeyError vs ValueError here
    with pytest.raises(KeyError):
        SomeFunnyOptions.parse_helpful_format("100: C")
